<?php

namespace App\Http\Controllers\Admin;

use App\Enums\StatusEnum;
use App\Enums\TicketReply;
use App\Enums\TicketStatus;
use App\Http\Controllers\Controller;
use App\Http\Services\TicketService;
use App\Jobs\SendEmailJob;
use App\Models\Admin;
use App\Models\AgentResponse;
use App\Models\AgentTicket;
use App\Models\SupportMessage;
use App\Models\SupportTicket;
use App\Rules\General\FileExtentionCheckRule;
use App\Rules\SanitizeHtml;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Carbon\Carbon;
use Illuminate\Validation\Rule;


class AdminTicketController extends Controller
{
    

    public $ticketService;

    public function __construct(TicketService $ticketService)
    {
        $this->ticketService = $ticketService;

    }

    /**
     * get ticket list
     */
     public function index(Request $request):Mixed{

        $title    = 'Ticket List';
        $tickets  = SupportTicket::with(['linkedPriority','messages'])->agent(request()->route('id'))->withCount(['messages' => function($q){
            $q->whereNull('admin_id')
            ->where('seen',StatusEnum::false->status());
        }])
        ->latest();


        $response =  ($this->ticketService->processData($tickets,$request));


        if($request->ajax()){
            
           $response =  ($this->ticketService->processData($tickets,$request));
           return json_encode($response);
        
        }
        return view('admin.ticket.index',compact('title'));
     }


    /**
     * mark  method
     */
    public function mark(Request $request) :RedirectResponse{

        $request->validate([
            'ticket_id'           => "required|array|exists:support_tickets,id",
            'short_note'          => "nullable|max:100",
            'priority_id'         => "nullable|exists:priorities,id",
            'assign'              => "nullable|exists:admins,id",
            'status'              =>  ["nullable",Rule::in(TicketStatus::toArray())],
        ],[
            "ticket_id.required"  => translate('Ticket Id Feild Is Required'),
            "ticket_id.array"     => translate('Ticket Id Feild Must Be An Array'),
            "ticket_id.exists"    => translate('Invalid Tickets Selected'),
        ]);

        $message                  = $this->ticketService->bulckAction($request);
       
        if(auth_user()->agent == StatusEnum::true->status()){
            return redirect()->route("admin.ticket.list")->with('success', $message );
        }
        return back()->with('success', $message );
    }



    /**
     * view ticket details
     */
    public  function view(string $id){

        $title   = 'View Ticket';
        $ticket  = SupportTicket::with(['agents','category' ,'messages','linkedPriority','admin','user'])
        ->agent()
        ->where('ticket_number',$id)
        ->firstOrfail();

        $previousTickets = SupportTicket::with(['agents','category' ,'messages','linkedPriority','admin','user'])
                            ->where('id','!=',$ticket->id)
                             ->when($ticket->user, function($query) use ($ticket) {    
                                $query->where("user_id",$ticket->user_id);
                             }, function ($query) use ($ticket) {
                                $query->where("email",$ticket->email);
                            })
                            ->agent()
                            ->latest()
                            ->take(5)
                            ->get();


        return view('admin.ticket.details',compact('title','ticket','previousTickets'));
    }

    /**
     * reply to a ticket 
     */
    public  function reply(Request $request) :RedirectResponse{


        $request->validate([
            'message'            => ['required' , new SanitizeHtml()],
            'redirect_to'         =>  ["required",Rule::in(TicketReply::toArray())],
            'ticket_id'          => 'required | exists:support_tickets,id',
            'draft_message_id'   => 'nullable | exists:support_messages,id',
            'files'              => [ new FileExtentionCheckRule(json_decode(site_settings('mime_types'),true),'Ticket File')],
            'status'             =>  ["nullable",Rule::in(TicketStatus::toArray())],
        ],[
            "message.required" => translate('Message Feild is Required'),
            "ticket_id.required" => translate('Ticket Id Feild Is Required'),
            "ticket_id.exists" => translate('Invalid Tickets Selected')
        ]);



        $ticket = SupportTicket::with(['user','messages'])->withCount('user')->where('id',$request->ticket_id)->first();

        if($request->status ){
            if($request->status == TicketStatus::SOLVED->value){
                if ((auth_user()->agent == StatusEnum::true->status()) && ($ticket->solved_request == SupportTicket::REJECTED || !$ticket->solved_request) ) {
                    $ticket->solved_request    = SupportTicket::REQUESTED;
                    $ticket->solved_request_at = Carbon::now();
                    $ticket->requested_by      = auth_user()->id;
                    $this->ticketService->notifyAdmin($ticket);
                    
                } else {
                    $ticket->solved_at = $ticket->solved_at ? $ticket->solved_at : Carbon::now() ;
                }
            }

            $ticket->status = $request->status ;
            $ticket->save();
        
        }

        $ticket_files = [];
        if($request->has('files')){

            try {
                $ticket_files = $this->ticketService->storeFiles( $request['files']);
            } catch (\Throwable $th) {
                return back()->with('error', translate("Unable to upload file. Check directory permissions"));
            }
            
        }

        $supportMessage = SupportMessage::firstOrCreate(
            ['id' => $request->input("draft_message_id")],
            ['support_ticket_id' => $request->ticket_id]
        );

        $supportMessage->support_ticket_id  = $request->ticket_id;
        $supportMessage->admin_id           = auth_user()->id;
        $supportMessage->message            = $request->message;
        $supportMessage->file               = json_encode($ticket_files);
        $supportMessage->is_draft           = StatusEnum::false->status();
        $supportMessage->created_at         = Carbon::now();
        $supportMessage->save();
        $status                             = 'success';
        $message                            = translate("Replied Successfully");


        if(auth_user()->agent == StatusEnum::true->status()){

            $this->calculateResponseTime($ticket,$supportMessage);

            $superAdmin = Admin::where('agent',StatusEnum::false->status())->first();

            AgentTicket::where('agent_id',"!=",$superAdmin->id)->where(function($q) use($request) {
                $q->where('agent_id',"!=",auth_user()->id)->where('ticket_id', $request->ticket_id);
            })->delete();
            
            $route = route('admin.ticket.view',$ticket->ticket_number);
            $notifyMessage = translate('Hello Dear!!! ').auth_user()->name. "Just Replied To a Ticket";
            $mailCode = [
                "role"           => 'Agent',
                "name"           => auth_user()->name,
                "ticket_number"  => $ticket->ticket_number,
                "link"           => $route
            ];
            $this->ticketService->notifyAgent($ticket,null, $notifyMessage ,"agent_ticket_reply" ,$route ,"TICKET_REPLY",
            $mailCode);
        }

        if($ticket->user_count  > 0){

            $route = route('user.ticket.view',$ticket->ticket_number);
            $notifyMessage = translate('Hello Dear!!! ').auth_user()->name. "  Just Replied To a Ticket";
            $mailCode = [
                "role"          => auth_user()->agent == StatusEnum::true->status() ? "Agent" :"SuperAdmin",
                "name"          => auth_user()->name,
                "ticket_number" => $ticket->ticket_number,
                "link"          => $route
            ];

            $this->ticketService->notifyUser($ticket->user,$notifyMessage,"ticket_reply",'TICKET_REPLY',$mailCode,$route);
        }
        else{

            $messageCount = SupportMessage::where("support_ticket_id",$ticket->id)
            ->whereNotNull('admin_id')
            ->count();

            if($messageCount  == 1){
                $code = [
                    'ticket_number' => $ticket->ticket_number ,
                    'link'          => route('ticket.reply',$ticket->ticket_number)
                ];
                SendEmailJob::dispatch($ticket, "SUPPORT_TICKET_REPLY" ,$code);
            }
          
        }

    
        $this->updateMessageStatus($ticket);





        if(TicketReply::TICKET_LIST->value == $request->input("redirect_to")){
            return redirect()->route('admin.ticket.list');
        }
        
        if(TicketReply::NEXT_TICKET->value == $request->input("redirect_to")){

            $nextTicket =  SupportTicket::agent()->where("id",'!=',$ticket->id)->latest()->first();
            if( $nextTicket){
                return redirect()->route('admin.ticket.view', $nextTicket->ticket_number);   
            }
            return redirect()->route('admin.ticket.list');
        }

        return back()->with($status,$message);

    }


    public function updateMessageStatus(SupportTicket $ticket) :void{

       $messages = $ticket->messages->whereNull('admin_id')
                  ->where('seen', StatusEnum::false->status());

        foreach ($messages->chunk(50) as $chunckMessages) {

            foreach ($chunckMessages as $message){
                $message->update(['seen' => StatusEnum::true->status()]);
            }
        }

    }



    /**
     * Cal Response time
     *
     * @param  $ticket
     * @param  $supportMessage
     * @return void
     */
    public function calculateResponseTime($ticket,$supportMessage){

     
        $messageCounts   = SupportMessage::where("support_ticket_id",$ticket->id)
                         ->where("admin_id",auth_user()->id)
                         ->count();

       if($messageCounts == 1){

            $createdAt                = Carbon::parse($ticket->created_at); 
            $now                      = Carbon::parse($supportMessage->created_at);
            $secondsDifference        = $now->diffInSeconds($createdAt);
            
            $hoursDifference          = floor($secondsDifference / 3600); 
            $minutesDifference        = round(($secondsDifference % 3600) / 60, 2); 
            
            $timeDifference           = $hoursDifference + ($minutesDifference / 100);

            $response                 = new AgentResponse();
            $response->agent_id       = auth_user()->id;
            $response->ticket_id      = $ticket->id;
            $response->response_time  = $timeDifference;
            $response->save();
       }


    }

    

   /**
    * update a ticket status
    */
    public function statusUpdate(Request $request) :RedirectResponse{

        $request->validate([
            'id'               => "required |exists:support_tickets,id",
            'status'           => "required",
            'key'              => "required",
        ],[
            'id.required'      => translate('Id Is Required'),
            'id.exists'        => translate('Select Id Is Invalid'),
            'status.required'  => translate('Status Is Required'),
            'key.required'     => translate('key Is Required'),
        ]);

        $status  = 'error';
        $message = translate('Invalid Key!! Cant Be Updated');
        try {

            $ticket                  =  SupportTicket::where('id',$request->id)->first();
            $ticket->{$request->key} =  $request->status;


            if ($request->key == "status" && $request->status == TicketStatus::SOLVED->value) {

                if (auth_user()->agent == StatusEnum::true->status() && ($ticket->solved_request == SupportTicket::REJECTED || !$ticket->solved_request)) {
                   
                    $ticket->solved_request    = SupportTicket::REQUESTED;
                    $ticket->solved_request_at = Carbon::now();
                    $ticket->requested_by      = auth_user()->id;
                    $this->ticketService->notifyAdmin($ticket);
                    
                    
                } else {
                    $ticket->solved_at = $ticket->solved_at ? $ticket->solved_at : Carbon::now() ;
                }
            }

            $ticket->save();
            $status  = 'success';
            $message = translate('Status Updated');

        } catch (\Throwable $ex) {

     
            return back()->with('error', translate("Some Thing Went Wrong!! Faild to update status"));
        }

    
        return back()->with( $status, $message );
    }



   /**
    * download a file 
    */
    public function download(Request $request) {

        $request->validate([
            'name'=>'required',
        ]);
        $url =  download_file(getFilePaths()['ticket']['path'],$request->name);
        if(!$url){
            return back()->with('error',translate('File Not Found'));
        }
        return $url;

    }


    /**
     * delete a specific file form ticket
     */

    public function deleteFile(Request $request) :RedirectResponse{
        
        $request->validate([
            'id'   => 'required',
            'name' => 'required',
        ]);
        $message      = translate("File Not Found");
        $status       = 'error';
        $ticketsFiles = SupportMessage::where('id',$request->id)->first();
        if(!$ticketsFiles){
            return back()->with($status,$message);
        }
        $files         = json_decode($ticketsFiles->file,true);
        $filteredFiles = array_filter($files, function ($item) use($request) {
            return $item !== $request->name;
        });
        $response = remove_file(getFilePaths()['ticket']['path'],$request->name);
        if($response){
            $ticketsFiles->file =  json_encode($filteredFiles);
            $ticketsFiles->save();
            $message            = translate("Deleted Successfully");
            $status             = 'success';
        }
        return back()->with($status,$message);
    }

    /**
     * export tickets 
     */

    public function export($extension) {

        $ticketData   =  SupportTicket::agent()->latest();
        $view         = null;
        if($extension == 'pdf'){
            $view     = "export_pdf";
        }
        return $this->ticketService->exportData($extension,$ticketData,$view );
    }


    /**
     * delete a specific ticket
     */

    public function delete(int | string $id) :RedirectResponse{

        $this->ticketService->deleteTicket((array)$id);
        $status  = 'success';
        $message = translate("Deleted Successfully");
        return redirect()->route('admin.ticket.list')->with($status,$message);
    }


    
    public function deleteMessage(int | string $id) :RedirectResponse{

        $message = SupportMessage::where('id',$id)->first();

        $status           = 'error';
        $alertMessage     = translate("Message Not Found");
        if($message){
            $status       = 'success';
            $alertMessage = translate("Deleted Successfully");
            $ticketFiles  = json_decode($message->file,true);
            $this->ticketService->unlinkFile($ticketFiles ? $ticketFiles :[] , getFilePaths()['ticket']['path']);
            $message->delete();
            
        }

        return back()->with($status,$alertMessage);
    }


    /**
     * Make a ticket mute and unmute
     * 
     * @param string | int id
     */
     public function makeMute(int | string $ticket_number) :RedirectResponse{


        $message     = translate('Ticket Muted');
        $admin       = auth_user();

        $ticket      = SupportTicket::where('ticket_number',$ticket_number)->firstOrfail();

        $mutedTicket = (array)$admin->muted_ticket;

        if(in_array($ticket->id, $mutedTicket)){
            $message     = translate('Ticket Unmuted');
            $mutedTicket = array_values(array_filter($mutedTicket, function($id) use ($ticket) {
                return $id !== $ticket->id;
            }));
        }
        else{
            array_push($mutedTicket, $ticket->id);
        }

        $admin->muted_ticket = $mutedTicket;
        $admin->save();


        return back()->with('success',$message );

     }



    /**
     * Ticket message update 
     * 
     * @param Request $request
     */
    public function updateMessage(Request $request) :RedirectResponse{

        $request->validate([
            'id'       => "required|exists:support_messages",
            'message'  => "required|string",
        ]);

        $message                     = SupportMessage::where("id",$request->input('id'))
                                        ->firstOrfail();
        $originalMassge              = $message->message;
        $message->message            = $request->input('message');
        $message->original_message	 = $originalMassge;
        $message->save();
        
        return back()->with('success',translate("Message Updated"));

    }



     /**
     * Ticket message update 
     * 
     * @param Request $request
     */
    public function saveDraft(Request $request) :array{

        $request->validate([
            'ticket_id'          => "required|exists:support_tickets,id",
            'draft'              => "required|string",
            'draft_id'           => 'nullable | exists:support_messages,id',

        ]);

        $status = false;
        $ticket = SupportTicket::where('id',$request->input('ticket_id'))->first();


        if($ticket && strip_tags($request->input('draft'),'<img>')){
            $admin       = auth_user();
            $status = true;
            $message                      = new SupportMessage();

            $message = SupportMessage::firstOrCreate(
                ['id' => $request->input("draft_id")],
                ['support_ticket_id' => $request->ticket_id]
            );

            $message->message             = $request->input('draft');
            $message->support_ticket_id   = $request->input('ticket_id');
            $message->admin_id            = $admin->id;
            $message->is_draft            = StatusEnum::true->status();
            $message->save();
        }

        return [
            'status'=>   $status ,
        ];
        

    }
}
