<?php

namespace App\Http\Controllers\User;

use App\Enums\NotifyStatus;
use App\Enums\StatusEnum;
use App\Http\Controllers\Admin\AgentController;
use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\CustomNotifications;
use App\Models\SupportTicket;
use App\Rules\General\FileExtentionCheckRule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\Rule;
use Illuminate\Contracts\View\View;

class HomeController extends Controller
{

    /** 
     * use dashboard
     */
    public function dashboard(Request $request) :View
    {
    
        $title = "User dashboard";
        $data  = $this->getDashboardData($request);
        return view('user.dashboard', compact('title','data'));
    }



    /**
     * get dashboard data
     */

     public function getDashboardData($request =  null) :array{


        $data['total_tickets']            = SupportTicket::where('user_id',auth_user('web')->id)->count();
        $data['total_pending_tickets']    = SupportTicket::where('user_id',auth_user('web')->id)->pending()->count();
        $data['total_processign_tickets'] = SupportTicket::where('user_id',auth_user('web')->id)->processing()->count();
        $data['total_solved_tickets']     = SupportTicket::where('user_id',auth_user('web')->id)->solved()->count();
        $data['total_closed_tickets']     = SupportTicket::where('user_id',auth_user('web')->id)->closed()->count();

        $data['ticket_by_category']       = Category::withCount(['tickets as tickets_count' => function($q){
            return $q->where('user_id',auth_user('web')->id);
        }])
        ->orderBy('tickets_count','desc')
        ->having("tickets_count" ,">", 0)
        ->pluck("tickets_count",'name',)
        ->mapWithKeys(function ($count, $name) {

            return [limit_words(get_translation($name),15) => $count];
        })
        ->toArray();

        $data['ticket_by_year']           = sortByMonth( SupportTicket::where('user_id',auth_user('web')->id)->selectRaw("MONTHNAME(created_at) as months, count(*) as total")
        ->whereYear('created_at', '=',date("Y"))
        ->groupBy('months')
        ->pluck('total', 'months')
        ->toArray());

        $data['latest_ticket']            =  SupportTicket::with(['user'])->where('user_id',auth_user('web')->id)->filter($request)->latest()->take(5)->get();
        return $data;

     }


    public function profile() :View
    {
        $title   = "User Profile";
        $user    = auth()->user();
        return view('user.profile', compact('title', 'user'));
    }


   /**
     * update user profile.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function profileUpdate(Request $request) :\Illuminate\Http\RedirectResponse
    {
        $user = auth_user('web');
        $request->validate([
            'name'    => 'required|max:255',
            'email'   => 'required|max:255|unique:users,email,'. $user->id,
            'image'   => ["image",new FileExtentionCheckRule(json_decode(site_settings('mime_types'),true))],
            'address' =>  Rule::requiredIf(site_settings("auto_ticket_assignment") == '1' && site_settings('geo_location') == 'map_base'),

        ],[
            'name.required'   => translate('Name Feild Is Required'),
            'email.required'  => translate('Email Feild Is Required'),
            'email.unique'    => translate('Email Must Be Unique'),
        ]);

        $address_data    = (new AgentController())->get_address($request);
        $user->name      = $request->name;
        $user->phone     = $request->phone;
        $user->email     = $request->email;
        $user->address   =  json_encode($address_data);
        $user->longitude =  isset($address_data['lon']) ?  $address_data['lon'] : null;
        $user->latitude	 =  isset($address_data['lat']) ?  $address_data['lat'] : null;
        if($request->hasFile('image')){
            try{
                $removefile  = $user->image ?: null;
                $user->image = storeImage($request->file('image'), getFilePaths()['profile']['user']['path'], null, $removefile);
            }catch (\Exception $exp){
                return back()->with('error',translate('File Upload Error: Insufficient Permissions Detected. Please Review Directory Access Permissions and Attempt the Upload Again'));

            }
        }

        $user->save();
        return back()->with('success',translate('Your profile has been updated.'));
    }




     /**
     * update admin password.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function passwordUpdate(Request $request) :\Illuminate\Http\RedirectResponse
    {
        $request->validate([
            'current_password'          => 'required',
            'password'                  => 'required|confirmed|min:5',
        ],
        [
            'current_password.required' => translate('Your Current Password is Required'),
            'password'                  => translate('Password Feild Is Required'),
            'password.confirmed'        => translate('Confirm Password Does not Match'),
            'password.min'              => translate('Minimum 5 digit or character is required'),
        ]);
        $user = auth_user('web');
        if (!Hash::check($request->current_password, $user->password)) {
            return back()->with('error', translate("Your Current Password does not match !!"));
        }
        $user->password = Hash::make($request->password);
        $user->save();
        return back()->with('success', translate("Password changed successfully"));
    }

    

    /**
     * get user notifications settings
     *
     * @return View
     */
    public function notificationSettings() :View {
        $title = "User Notifications Settings";
        return view('user.notification_settings', compact('title'));
    }

    /**
     * Update notifiation settings
     *
     * @param Request $request
     * @return \Illuminate\Http\RedirectResponse
     */
    public function updateNotificationSettings(Request $request) :\Illuminate\Http\RedirectResponse {

        $user_notifications = user_notification();
        if($request->user_notificatoon){
            $request_data = $request->user_notificatoon;
            foreach($user_notifications  as $key=>$setting){
                if(isset($request_data[$key])){
                    foreach($setting as  $sub_key=>$val){
                        if(isset($request_data[$key][$sub_key])){
                            $setting[$sub_key] =  $request_data[$key][$sub_key];
                        }
                    }
                    $user_notifications[$key] = $setting;
                }
            }
        }
        auth_user('web')->notification_settings =  json_encode( $user_notifications);
        auth_user('web')->save();   
        return back()->with('success',translate('Notifications Settings Updated'));
    }

    /**
     * read a notifications
     */

    public function readNotification(Request $request) :string{

        $user_notification = CustomNotifications::where("id",$request->id)
        ->where("notify_id",auth_user('web')->id)
        ->where('notification_for',NotifyStatus::USER)->first();
        $status  = false;
        $message = translate('Notification Not Found');
        if( $user_notification ){
            $user_notification->is_read =  (StatusEnum::true)->status();
            $user_notification->save();
            $status  = true;
            $message = translate('Notification Readed');
        }
        return json_encode([
            "status"  => $status,
            "message" => $message
        ]);

    }



    /**
     * view all notifications
     */

     public function notification(Request $request) :View{

        $title  = "All Notifications";
        $layout = "user.layouts.master";
        CustomNotifications::where("notify_id",auth_user('web')->id)
        ->where('is_read' ,(StatusEnum::false)->status())
        ->where('notification_for',NotifyStatus::USER)->update([
            "is_read" =>  (StatusEnum::true)->status()
        ]);
        
        $notifications =  CustomNotifications::where("notify_id",auth_user('web')->id)
        ->where('is_read' ,(StatusEnum::true)->status())
        ->where('notification_for',NotifyStatus::USER)->latest()->paginate(paginateNumber());
        
        return view('notification',compact('title','notifications','layout'));
     }

     /**
      * delete a specific notification
      */
     public function deleteNotification(int | string $id) :\Illuminate\Http\RedirectResponse {

        $notification =  CustomNotifications::where('id',$id)
        ->where("notify_id",auth_user('web')->id)
        ->where('notification_for',NotifyStatus::USER)
        ->first();
        $status      = 'error';
        $message     = translate('Notification Not Found');
        if($notification){
            $status  = 'success';
            $message = translate('Notification Deleted');
            $notification->delete();
        }
        return back()->with($status,$message);
     }




    


}
